<?php
header("Content-Type: application/json; charset=UTF-8");
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, GET, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type");

require 'config.php'; // Use the PDO connection from config.php

/**
 * Parses a timestamp string from known formats into 'Y-m-d H:i:s'.
 *
 * @param string $timestampStr The timestamp string from the device.
 * @return string|false The formatted timestamp string or false on failure.
 */
function parseTimestamp($timestampStr) {
    if (empty($timestampStr)) {
        return false;
    }
    // Try parsing 24-hour format first
    $dateTime = DateTime::createFromFormat('Y-m-d H:i:s', $timestampStr);
    if ($dateTime !== false) {
        return $dateTime->format('Y-m-d H:i:s');
    }
    // Try parsing 12-hour format with AM/PM
    $dateTime = DateTime::createFromFormat('Y-m-d h:i:s A', $timestampStr);
    if ($dateTime !== false) {
        return $dateTime->format('Y-m-d H:i:s');
    }
    // Add more formats here if needed
    // $dateTime = DateTime::createFromFormat('...', $timestampStr);
    // if ($dateTime !== false) { return $dateTime->format('Y-m-d H:i:s'); }

    // If all formats fail, try strtotime as a fallback (less reliable)
    $timestamp = strtotime($timestampStr);
    if ($timestamp !== false) {
        return date('Y-m-d H:i:s', $timestamp);
    }

    return false; // Could not parse
}

// --- Main Script ---

$inputJson = file_get_contents("php://input");
$inputData = json_decode($inputJson, true);

$locationsToProcess = [];
$response = ["status" => "error", "message" => "No valid data received.", "processed" => 0, "failed" => 0];

if (!$inputData) {
    error_log("add_location: Invalid or empty JSON received.");
    echo json_encode($response);
    exit;
}

// Check if data is a batch under 'locations' key or a single object
if (isset($inputData['locations']) && is_array($inputData['locations'])) {
    // Batch data
    $locationsToProcess = $inputData['locations'];
    error_log("add_location: Received batch data with " . count($locationsToProcess) . " records.");
} elseif (isset($inputData['device_id'])) {
    // Single data object
    $locationsToProcess[] = $inputData;
    error_log("add_location: Received single data record.");
} else {
    error_log("add_location: JSON structure not recognized. Payload: " . $inputJson);
    $response["message"] = "JSON structure not recognized.";
    echo json_encode($response);
    exit;
}

if (empty($locationsToProcess)) {
    error_log("add_location: No location records found to process.");
    $response["message"] = "No location records found to process.";
    echo json_encode($response);
    exit;
}

// Prepare the SQL statement once
// IMPORTANT: Ensure your 'locations' table has columns for accuracy, altitude, heading and they allow NULLs.
$sql = "INSERT INTO locations (device_id, device_name, latitude, longitude, timestamp, status, battery, accuracy, altitude, heading)
        VALUES (:device_id, :device_name, :latitude, :longitude, :timestamp, :status, :battery, :accuracy, :altitude, :heading)";

try {
    $stmt = $conn->prepare($sql);
    $processedCount = 0;
    $failedCount = 0;

    // Set default timezone for timestamp parsing fallback (strtotime)
    date_default_timezone_set("Asia/Baghdad");

    foreach ($locationsToProcess as $index => $location) {
        // Validate required fields for each location object
        $required_fields = ["device_id", "device_name", "latitude", "longitude", "status", "timestamp"];
        $missing_fields = [];
        foreach ($required_fields as $field) {
            if (!isset($location[$field]) || $location[$field] === '') {
                $missing_fields[] = $field;
            }
        }

        if (!empty($missing_fields)) {
            error_log("add_location: Record " . ($index + 1) . " skipped. Missing required fields: " . implode(', ', $missing_fields) . ". Data: " . json_encode($location));
            $failedCount++;
            continue; // Skip this record
        }

        // Parse and validate timestamp
        $parsedTimestamp = parseTimestamp($location["timestamp"]);
        if ($parsedTimestamp === false) {
            error_log("add_location: Record " . ($index + 1) . " skipped. Could not parse timestamp: '" . $location["timestamp"] . "'. Data: " . json_encode($location));
            $failedCount++;
            continue; // Skip this record
        }

        // Assign data, handling optional fields
        $device_id   = $location["device_id"];
        $device_name = $location["device_name"];
        $latitude    = $location["latitude"];
        $longitude   = $location["longitude"];
        $status      = $location["status"];
        $timestamp   = $parsedTimestamp; // Use the parsed and formatted timestamp

        // Optional fields - assign null if not present
        $battery     = isset($location["battery"]) && $location["battery"] !== '' ? $location["battery"] : null;
        $accuracy    = isset($location["accuracy"]) && $location["accuracy"] !== '' ? $location["accuracy"] : null;
        $altitude    = isset($location["altitude"]) && $location["altitude"] !== '' ? $location["altitude"] : null;
        $heading     = isset($location["heading"]) && $location["heading"] !== '' ? $location["heading"] : null;

        // Bind parameters for the current record
        $stmt->bindParam(':device_id', $device_id);
        $stmt->bindParam(':device_name', $device_name);
        $stmt->bindParam(':latitude', $latitude);
        $stmt->bindParam(':longitude', $longitude);
        $stmt->bindParam(':timestamp', $timestamp);
        $stmt->bindParam(':status', $status);
        $stmt->bindParam(':battery', $battery);
        $stmt->bindParam(':accuracy', $accuracy);
        $stmt->bindParam(':altitude', $altitude);
        $stmt->bindParam(':heading', $heading);

        // Execute the insert for the current record
        if ($stmt->execute()) {
            $processedCount++;
        } else {
            $failedCount++;
            // Log the specific PDO error for this record
            $errorInfo = $stmt->errorInfo();
            error_log("add_location: Failed to insert record " . ($index + 1) . ". PDO Error: [" . $errorInfo[0] . "] " . $errorInfo[2] . ". Data: " . json_encode($location));
        }
    } // End foreach loop

    // Final response
    if ($processedCount > 0 && $failedCount == 0) {
        $response["status"] = "success";
        $response["message"] = "Successfully processed " . $processedCount . " location records.";
    } elseif ($processedCount > 0 && $failedCount > 0) {
        $response["status"] = "partial_success";
        $response["message"] = "Processed " . $processedCount . " records successfully, but " . $failedCount . " records failed.";
    } elseif ($processedCount == 0 && $failedCount > 0) {
        $response["status"] = "error";
        $response["message"] = "Failed to process all " . $failedCount . " records.";
    } else {
         // Should not happen if input validation worked, but as a fallback
         $response["status"] = "error";
         $response["message"] = "No records were processed.";
    }
    $response["processed"] = $processedCount;
    $response["failed"] = $failedCount;

} catch (PDOException $e) {
    // Catch errors during prepare() or general connection issues
    error_log("add_location: Database Error: " . $e->getMessage());
    $response["status"] = "error";
    $response["message"] = "Database error occurred during processing.";
    // Avoid exposing detailed DB errors in production response
    // $response["db_error"] = $e->getMessage();
} finally {
    $conn = null; // Close connection
}

echo json_encode($response, JSON_UNESCAPED_UNICODE);
?>
