<?php
require 'config.php';

header('Content-Type: application/json; charset=UTF-8');
header("Access-Control-Allow-Origin: *");

$device_id = $_GET['device_id'] ?? null;
if (!$device_id) {
    http_response_code(400); // Bad Request
    echo json_encode(["status" => "error", "message" => "معرف الجهاز مطلوب."]); // Simpler message
    exit();
}

try {
    // Check if the database connection is valid before proceeding
    if (!$conn) {
        // This case should ideally not be reached if config.php dies on connection failure,
        // but it's good practice for robustness and satisfies the linter.
        http_response_code(500); // Internal Server Error
        echo json_encode([
            "status" => "error",
            "message" => "خطأ في الاتصال بقاعدة البيانات."
        ], JSON_UNESCAPED_UNICODE);
        exit();
    }

    // Assumes 'timestamp' column is now DATETIME or TIMESTAMP type
    $query = "
        SELECT
            device_id,
            device_name,
            latitude,
            longitude,
            timestamp, -- Select the native timestamp
            battery
        FROM locations
        WHERE device_id = :device_id
        ORDER BY timestamp DESC -- Direct sorting on the native timestamp column
        LIMIT 1
    ";
    $stmt = $conn->prepare($query);
    $stmt->execute([':device_id' => $device_id]);
    $data = $stmt->fetch(PDO::FETCH_ASSOC);

    if ($data) {
        // Format the timestamp for output if needed, e.g., using PHP's date functions
        // Example: $data['timestamp_formatted'] = date('Y-m-d h:i:s A', strtotime($data['timestamp']));
        http_response_code(200); // OK
        echo json_encode(["status" => "success", "data" => $data], JSON_UNESCAPED_UNICODE);
    } else {
        http_response_code(404); // Not Found
        echo json_encode(["status" => "error", "message" => "لا توجد بيانات متاحة لهذا الجهاز."], JSON_UNESCAPED_UNICODE);
    }
} catch (PDOException $e) {
    // Log the detailed error to a file for the admin/developer
    error_log("Database Error in get_latest_location.php for device_id $device_id: " . $e->getMessage() . "\n", 3, "log.txt");

    // Send a generic error message
    http_response_code(500); // Internal Server Error
    echo json_encode(["status" => "error", "message" => "حدث خطأ أثناء معالجة طلبك."], JSON_UNESCAPED_UNICODE);
} finally {
    // Ensure connection is closed, check if $conn was successfully initialized
    if ($conn) {
       $conn = null;
    }
}
?>
