<?php
require 'config.php';

header("Content-Type: application/json; charset=UTF-8");

// استلام المعاملات من الـ GET
$device_id = $_GET['device_id'] ?? null;
$date      = $_GET['date'] ?? null;       // Expected format: YYYY-MM-DD
$time_from = $_GET['time_from'] ?? null; // Expected format: HH:MM:SS (24-hour)
$time_to   = $_GET['time_to'] ?? null;   // Expected format: HH:MM:SS (24-hour)
$page      = isset($_GET['page']) ? (int)$_GET['page'] : 1; // Page number for pagination
$limit     = isset($_GET['limit']) ? (int)$_GET['limit'] : 50; // Records per page (Reduced default limit to 50)

// Calculate OFFSET for pagination
$offset = ($page - 1) * $limit;

// --- Database Index Recommendations ---
// For optimal performance, ensure you have indexes on the 'locations' table:
// 1. On the 'device_id' column:
//    CREATE INDEX idx_locations_device_id ON locations (device_id);
// 2. On the 'timestamp' column:
//    CREATE INDEX idx_locations_timestamp ON locations (timestamp);
// 3. A composite index for common filtering/sorting combinations:
//    CREATE INDEX idx_locations_device_timestamp ON locations (device_id, timestamp);
// --- End Index Recommendations ---

// Build the base query
// Assuming 'timestamp' column is now DATETIME
$query = "
    SELECT
        id, device_id, device_name, latitude, longitude, timestamp, status, battery
    FROM
        locations
    WHERE 1=1
"; // Using 1=1 to simplify appending conditions

$params = [];

// تصفية حسب الجهاز
if ($device_id) {
    $query .= " AND device_id = :device_id";
    $params[':device_id'] = $device_id;
}

// تصفية حسب التاريخ والوقت (Using efficient range queries)
if ($date) {
    $start_datetime = $date . ' ' . ($time_from ?: '00:00:00');
    $end_datetime   = $date . ' ' . ($time_to ?: '23:59:59');

    // Filter using direct string comparison (assuming 'YYYY-MM-DD HH:MM:SS' format)
    // This might be faster if the format is consistent and indexed.
    // IMPORTANT: Verify this works correctly with your exact stored format.
    $query .= " AND timestamp >= :start_datetime
                  AND timestamp <= :end_datetime";
    $params[':start_datetime'] = $start_datetime;
    $params[':end_datetime'] = $end_datetime;
}
// Note: Removed the elseif block for time_from/time_to without date as it's less common
// and requires clearer definition if needed.

// Add ORDER BY using STR_TO_DATE
// IMPORTANT: Adjust '%Y-%m-%d %H:%i:%s' if your stored format is different!
$query .= " ORDER BY STR_TO_DATE(timestamp, '%Y-%m-%d %H:%i:%s') ASC"; // Order chronologically

// Add Pagination
$query .= " LIMIT :limit OFFSET :offset";
$params[':limit'] = $limit;
$params[':offset'] = $offset;


// Prepare and execute the query
try {
    $stmt = $conn->prepare($query);

    // Bind parameters dynamically based on what's included
    foreach ($params as $key => &$val) {
        // Determine data type for binding (PDO::PARAM_INT or PDO::PARAM_STR)
        $param_type = (is_int($val) || $key === ':limit' || $key === ':offset') ? PDO::PARAM_INT : PDO::PARAM_STR;
        $stmt->bindParam($key, $val, $param_type);
    }
    unset($val); // Unset reference

    $stmt->execute();
    $locations = $stmt->fetchAll(PDO::FETCH_ASSOC);

    // --- Get total count for pagination ---
    // Build the count query based on the filtering conditions applied
    $countQuery = "SELECT COUNT(*) FROM locations WHERE 1=1";
    $countParams = []; // Separate params for count query

    if ($device_id) {
        $countQuery .= " AND device_id = :device_id";
        $countParams[':device_id'] = $device_id;
    }
    if ($date) {
        // Filter count using direct string comparison (assuming 'YYYY-MM-DD HH:MM:SS' format)
        // IMPORTANT: Verify this works correctly with your exact stored format.
        $countQuery .= " AND timestamp >= :start_datetime
                         AND timestamp <= :end_datetime";
        // Use the already calculated start/end datetime strings
        $countParams[':start_datetime'] = $start_datetime;
        $countParams[':end_datetime'] = $end_datetime;
    }
    // Note: No need to include ORDER BY or LIMIT/OFFSET params for count

    $countStmt = $conn->prepare($countQuery);
    // Bind parameters for the count query
    foreach ($countParams as $key => &$val) {
         $param_type = is_int($val) ? PDO::PARAM_INT : PDO::PARAM_STR;
         $countStmt->bindParam($key, $val, $param_type);
    }
    unset($val);

    $countStmt->execute();
    $totalRecords = (int)$countStmt->fetchColumn();
    $totalPages = ($limit > 0) ? ceil($totalRecords / $limit) : 0; // Avoid division by zero if limit is 0


    // Return results along with pagination info
    http_response_code(200); // OK
    echo json_encode([
        "status" => "success",
        "data" => $locations,
        "pagination" => [
            "currentPage" => $page,
            "totalPages" => $totalPages,
            "totalRecords" => $totalRecords,
            "limit" => $limit
        ]
    ], JSON_UNESCAPED_UNICODE);

} catch (PDOException $e) {
    // Log the detailed error to a file
    // Construct a more informative log message
    $log_message = "Database Error in get_locations.php: " . $e->getMessage();
    $log_message .= " | Parameters: " . json_encode($_GET); // Log received parameters
    error_log($log_message . "\n", 3, "log.txt");

    // Send a generic error message to the client
    http_response_code(500); // Internal Server Error
    echo json_encode([
        "status" => "error",
        "message" => "حدث خطأ أثناء استرجاع بيانات المواقع."
        // "error_details" => $e->getMessage() // Optionally include for debugging ONLY, remove for production
    ], JSON_UNESCAPED_UNICODE);

} finally {
    $conn = null; // Ensure connection is closed
}
?>
